package facturation.logique;

import java.util.List;

import facturation.dao.ArticleDAO;
import facturation.dao.ClientDAO;
import facturation.dao.FactureDAO;
import facturation.modele.Article;
import facturation.modele.Client;

/**
 * Classe gérant la logique générale de l'application.
 * On pourrait scinder cette classe en plusieurs parties.
 * 
 * <p> On rencontre ici, comme dans la plupart des applications, le problème suivant:
 * L'interface doit-elle connaître les objets du modèle? Par exemple, quand l'interface 
 * veut afficher les données du client, elle doit connaître son nom, son adresse, son id...
 * 
 * <p>Première solution: facturationLogique renvoie des objets "Client". L'interface utilisateur 
 * peut accéder à leurs champs pour les afficher. Le problème est qu'il est tentant pour le programmeur de 
 * modifier lesdits objets, et de s'attendre à voir ces modifications prises en compte. On perd alors la belle architecture 
 * en couche que nous avons proposé. On peut résoudre ce problème en documentant l'architecture en question, mais 
 * c'est une solution qui reste à la merci des programmeurs. Il est aussi possible de renvoyer des objets non modifiables, 
 * mais cela implique a priori de dupliquer les classes du modèle... Certains programmeurs renvoient des copies des objets utilisés par FacturationLogique,
 * et non les objets eux-même. Bref, on dispose d'un certain nombre de solution, dont aucune n'est 
 * complètement satisfaisante.
 * 
 * <p> Deuxième solution: facturationLogique ne renvoie pas d'objet "client", mais donne accès à leurs champs. C'est une solution qui allourdit considérablement
 * la couche applicative (mais le code peut être généré automatiquement par eclipse). Elle est assez complexe à utiliser quand le résultat est composite (quand c'est, par exemple, 
 * une liste).
 * 
 * <p> Le langage C++ propose une solution intéressante, en ayant la notion d'objets "contants", i.e. non modifiables.
 * 
 * <p> Nous choisissons dans cette application de renvoyer directement les objets du modèle. ON IMPOSE AU PROGRAMMEUR DE NE PAS MODIFIER LESDITS OBJECTS DEPUIS L'INTERFACE.
 * TOUTE MODIFICATION DOIT PASSER PAR L'APPEL DE MÉTHODES SUR FACTURATIONLOGIQUE.
 *  
 * @author rosmord
 *
 */
public class FacturationLogique {
	private ClientDAO clientDAO;
	private FactureDAO factureDAO;
	private ArticleDAO articleDAO;
	
	private Client clientCourant = new Client();

	public FacturationLogique(ClientDAO clientDAO, ArticleDAO articleDAO) {
		this.clientDAO = clientDAO;
		this.articleDAO= articleDAO;
	}

	/**
	 * Prépare le travail sur un nouveau client.
	 */
	public void reinitialiserClient() {
		clientCourant= new Client();
	}
	
	public void fixerNomClient(String nom) {
		clientCourant.setNom(nom);
	}

	public void fixerAdresseClient(String adresse) {
		clientCourant.setAdresse(adresse);
	}

	public void creerClient() {
		clientCourant.setIdClient(clientDAO.getNextId());
		clientDAO.create(clientCourant);
	}
	
	
	public List<Client> chercherClientParNom(String nom) {
		return clientDAO.findClientByNom(nom);
	}


	public Client getClientCourant() {
		return clientCourant;
	}
	
	public List<Article> getCatalogue() {
		return null;
	}

	public void selectionnerClient(int id) {
		clientCourant= clientDAO.findClientById(id);
	}
}
